@@ -23,10 +23,6 @@
 #include <string.h>
 #include <glib/gi18n.h>
 
-#if !GTK_CHECK_VERSION (2, 14, 0)
-    #define gtk_dialog_get_content_area(dialog) dialog->vbox
-#endif
-
 struct _KatzePreferencesPrivate
 {
     #if HAVE_HILDON
